var aliases = {
  "AnimationEvent": "AnimationEvent",
  "BeforeUnloadEvent": "BeforeUnloadEvent",
  "CloseEvent": "CloseEvent",
  "CompositionEvent": "CompositionEvent",
  "CustomEvent": "CustomEvent",
  "DeviceMotionEvent": "DeviceMotionEvent",
  "DeviceOrientationEvent": "DeviceOrientationEvent",
  "DragEvent": "DragEvent",
  "ErrorEvent": "ErrorEvent",
  "Event": "Event",
  "Events": "Event",
  "FocusEvent": "FocusEvent",
  "HashChangeEvent": "HashChangeEvent",
  "HTMLEvents": "Event",
  "IDBVersionChangeEvent": "IDBVersionChangeEvent",
  "KeyboardEvent": "KeyboardEvent",
  "MessageEvent": "MessageEvent",
  "MouseEvent": "MouseEvent",
  "MouseEvents": "MouseEvent",
  "PageTransitionEvent": "PageTransitionEvent",
  "PopStateEvent": "PopStateEvent",
  "StorageEvent": "StorageEvent",
  "SVGEvents": "Event",
  "TextEvent": "CompositionEvent",
  "TouchEvent": "TouchEvent",
  "TrackEvent": "TrackEvent",
  "TransitionEvent": "TransitionEvent",
  "UIEvent": "UIEvent",
  "UIEvents": "UIEvent",
  "WebGLContextEvent": "WebGLContextEvent",
  "WheelEvent": "WheelEvent",
};
